<?php

namespace Drupal\proc\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\Plugin\Field\FieldFormatter\EntityReferenceLabelFormatter;

/**
 * Plugin implementation of the 'entity reference label' formatter.
 *
 * @FieldFormatter(
 *   id = "proc_entity_reference_armoured_formatter",
 *   label = @Translation("Armoured"),
 *   description = @Translation("Display the armoured cipher text for decryption on lists."),
 *   field_types = {
 *     "proc_entity_reference_field"
 *   }
 * )
 */
class ProcArmouredFieldFormatter extends EntityReferenceLabelFormatter {

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = [];

    $current_user_id = \Drupal::currentUser()->id();
    $keyring = _proc_get_keys($current_user_id, 'uid');

    $privkey = $keyring['encrypted_private_key'];
    $recipient = FALSE;

    foreach ($this->getEntitiesToView($items, $langcode) as $delta => $entity) {
      $recipients = $entity->get('field_recipients_set')->getValue();
      $label = $entity->label();

      $elements[$delta]['placeholder'] = [
        '#type' => 'inline_template',
        '#context' => [
          'label' => $label,
          'class' => 'proc-armoured-field-formatter',
        ],
        '#template' => '<div class="{{class}}">{{ label }}</div>',
      ];

      if (in_array($current_user_id, array_column($recipients, 'target_id'))) {
        $recipient = TRUE;
        $cipher_text = $entity->getCipherText();

        $elements[$delta]['placeholder']['#context']['value'] = $cipher_text;
        $elements[$delta]['placeholder']['#context']['pid'] = $entity->id();
        $elements[$delta]['placeholder']['#template'] = '<div id="proc-list-item-{{ pid }}" class="{{class}}" data-proc-list-item-pid="{{ pid }}" data-proc-list-item-cipher-text="{{ value|raw }}">{{ label }}</div>';
      }
    }

    if ($recipient) {
      $elements['#attached'] = [
        'library' => [
          0 => 'proc/openpgpjs',
          1 => 'proc/proc-decrypt-list',
        ],
        'drupalSettings' => [
          'proc' => [
            'proc_labels' => _proc_js_labels(),
            'proc_pass' => _proc_common_get_decryption_form_data(NULL)['proc_pass'],
            'proc_privkey' => $privkey,
            'proc_keyring_type' => $keyring['keyring_type'],
          ],
        ],
      ];
    }

    return $elements;
  }

}
